from typing import List

from rdflib import Graph, Literal

from ..ns import *
from .declarations.UserAccount import UserAccount
from .declarations.OnlineContactPoint import OnlineContactPoint
from .SocialMedia import SocialMedia
from .Post import Post


class UserAccount(UserAccount):
    __type__ = SM["UserAccount"]

    isAccountIssuedBy: SocialMedia = None
    isUserAccountOf: OnlineContactPoint = None
    URL: Literal = None
    userAccountName: Literal = None
    isCreatorOf: List[Post] = None

    def _addProperties(self, g: Graph):
        super()._addProperties(g)

        if self.isAccountIssuedBy:
            g.add((self.uriRef, SM["isAccountIssuedBy"],
                  self.isAccountIssuedBy.uriRef))

        if self.isUserAccountOf:
            g.add((self.uriRef, SM["isUserAccountOf"],
                  self.isUserAccountOf.uriRef))

        if self.URL:
            g.add((self.uriRef, SM["URL"], self.URL))

        if self.userAccountName:
            g.add((self.uriRef, SM["userAccountName"], self.userAccountName))

        if self.isCreatorOf:
            for isCreatorOf in self.isCreatorOf:
                g.add((self.uriRef, SM["isCreatorOf"], isCreatorOf.uriRef))



from typing import List

from rdflib import Graph, Literal

from ..ns import *
from .Organization import Organization
from .PrivateOrgActivityType import PrivateOrgActivityType


class PrivateOrganization(Organization):
    __type__ = COV["PrivateOrganization"]

    hasPrivateOrgActivityType: List[PrivateOrgActivityType] = None
    REANumber: Literal = None
    businessObjective: List[Literal] = None
    VATnumber: Literal = None
    startingActivityDate: Literal = None

    def _addProperties(self, g: Graph):
        super()._addProperties(g)

        if self.hasPrivateOrgActivityType:
            for hasPrivateOrgActivityType in self.hasPrivateOrgActivityType:
                g.add(
                    (self.uriRef, COV["hasPrivateOrgActivityType"], hasPrivateOrgActivityType.uriRef))

        if self.REANumber:
            g.add((self.uriRef, COV["REANumber"], self.REANumber))

        if self.businessObjective:
            for businessObjective in self.businessObjective:
                g.add(
                    (self.uriRef, COV["businessObjective"], businessObjective))

        if self.VATnumber:
            g.add((self.uriRef, COV["VATnumber"], self.VATnumber))

        if self.startingActivityDate:
            g.add(
                (self.uriRef, COV["startingActivityDate"], self.startingActivityDate))



from typing import List
from rdflib import Graph, URIRef

from .declarations.PointOfInterest import PointOfInterest
from ..ns import *
from .PointOfInterestCategory import PointOfInterestCategory
from ..clv.Address import Address


class PointOfInterest(PointOfInterest):
    __type__ = POI["PointOfInterest"]

    hasPOICategory: List[PointOfInterestCategory] = []
    hasAddress: List[Address] = []
    hasPOINameInITime = None
    hasImage = None
    atTime = None
    hasGeometry = None
    hasPOIState = None
    isIncludedInPOI = None
    POIofficialName = None
    POIdescription = None
    POIID = None

    def _addProperties(self, g: Graph):
        super()._addProperties(g)

        for poiCategory in self.hasPOICategory:
            g.add((self.uriRef, POI["hasPOICategory"], URIRef(
                POI_CLASSIFICATION[poiCategory])))

        for address in self.hasAddress:
            g.add((self.uriRef, CLV["hasAddress"], address.uriRef))



from typing import List

from rdflib import Graph, Literal

from ..clv.Address import Address
from ..l0.Location import Location
from ..ns import *
from .declarations.Person import Person
from .PersonTitle import PersonTitle
from .ResidenceInTime import ResidenceInTime
from .Sex import Sex


class Person(Person):
    __type__ = CPV["Person"]

    hasAddress: List[Address] = None
    hasParentalRelationshipWith: List[Person] = None
    isChildOf: List[Person] = None
    isConsortOf: List[Person] = None
    isGrandChildOf: List[Person] = None
    isGrandFatherOf: List[Person] = None
    isNephewOf: List[Person] = None
    isParentOf: List[Person] = None
    isUncleOf: List[Person] = None
    hasPersonTitle: List[PersonTitle] = None
    hasRelationshipWith: List[Person] = None
    knows: List[Person] = None
    hasResidenceInTime: List[ResidenceInTime] = None
    residentIn: List[Address] = None
    hasBirthPlace: Location = None
    hasCitizenship: List[Location] = None
    hasSex: Sex = None
    dateOfBirth: Literal = None
    personID: Literal = None
    altName: Literal = None
    familyName: Literal = None
    givenName: Literal = None
    taxCode: Literal = None

    def _addProperties(self, g: Graph):
        super()._addProperties(g)

        if self.hasAddress:
            for hasAddress in self.hasAddress:
                g.add((self.uriRef, CLV["hasAddress"], hasAddress.uriRef))

        if self.hasParentalRelationshipWith:
            for hasParentalRelationshipWith in self.hasParentalRelationshipWith:
                g.add(
                    (self.uriRef, CPV["hasParentalRelationshipWith"], hasParentalRelationshipWith.uriRef))

        if self.isChildOf:
            for isChildOf in self.isChildOf:
                g.add((self.uriRef, CPV["isChildOf"], isChildOf.uriRef))

        if self.isConsortOf:
            for isConsortOf in self.isConsortOf:
                g.add((self.uriRef, CPV["isConsortOf"], isConsortOf.uriRef))

        if self.isGrandChildOf:
            for isGrandChildOf in self.isGrandChildOf:
                g.add(
                    (self.uriRef, CPV["isGrandChildOf"], isGrandChildOf.uriRef))

        if self.isGrandFatherOf:
            for isGrandFatherOf in self.isGrandFatherOf:
                g.add(
                    (self.uriRef, CPV["isGrandFatherOf"], isGrandFatherOf.uriRef))

        if self.isNephewOf:
            for isNephewOf in self.isNephewOf:
                g.add((self.uriRef, CPV["isNephewOf"], isNephewOf.uriRef))

        if self.isParentOf:
            for isParentOf in self.isParentOf:
                g.add((self.uriRef, CPV["isParentOf"], isParentOf.uriRef))

        if self.isUncleOf:
            for isUncleOf in self.isUncleOf:
                g.add((self.uriRef, CPV["isUncleOf"], isUncleOf.uriRef))

        if self.hasPersonTitle:
            for hasPersonTitle in self.hasPersonTitle:
                g.add(
                    (self.uriRef, CPV["hasPersonTitle"], hasPersonTitle.uriRef))

        if self.hasRelationshipWith:
            for hasRelationshipWith in self.hasRelationshipWith:
                g.add(
                    (self.uriRef, CPV["hasRelationshipWith"], hasRelationshipWith.uriRef))

        if self.knows:
            for knows in self.knows:
                g.add((self.uriRef, CPV["knows"], knows.uriRef))

        if self.hasResidenceInTime:
            for hasResidenceInTime in self.hasResidenceInTime:
                g.add(
                    (self.uriRef, CPV["hasResidenceInTime"], hasResidenceInTime.uriRef))

        if self.residentIn:
            for residentIn in self.residentIn:
                g.add((self.uriRef, CPV["residentIn"], residentIn.uriRef))

        if self.hasCitizenship:
            for hasCitizenship in self.hasCitizenship:
                g.add(
                    (self.uriRef, CPV["hasCitizenship"], hasCitizenship.uriRef))

        if self.hasBirthPlace:
            g.add(self.uriRef, CPV["hasBirthPlace"], self.hasBirthPlace.uriRef)

        if self.dateOfBirth:
            g.add(self.uriRef, CPV["dateOfBirth"], self.dateOfBirth)

        if self.personID:
            g.add(self.uriRef, CPV["personID"], self.personID)

        if self.altName:
            g.add(self.uriRef, CPV["altName"], self.altName)

        if self.familyName:
            g.add(self.uriRef, CPV["familyName"], self.familyName)

        if self.givenName:
            g.add(self.uriRef, CPV["givenName"], self.givenName)

        if self.taxCode:
            g.add(self.uriRef, CPV["taxCode"], self.taxCode)



from rdflib import Graph, Literal

from ..ns import *
from ..l0.Object import Object
from .OnlineContactPoint import OnlineContactPoint


class WebSite(Object):
    __type__ = SM["WebSite"]

    isWebSiteOf: OnlineContactPoint = None
    URL: Literal = None

    def _addProperties(self, g: Graph):
        super()._addProperties(g)

        if self.isWebSiteOf:
            g.add((self.uriRef, SM["isWebSiteOf"], self.isWebSiteOf.uriRef))

        if self.URL:
            g.add((self.uriRef, SM["URL"], self.URL))



from typing import List

from rdflib import Graph, Literal

from ..ns import *
from .AddressComponent import AddressComponent


class AddressArea(AddressComponent):
    __type__ = CLV["AddressArea"]

    name: List[Literal] = None

    def _addProperties(self, g: Graph):
        super()._addProperties(g)

        if self.name:
            for name in self.name:
                g.add((self.uriRef, L0["name"], name))



from rdflib import URIRef

from ..ns import *
from ..l0.Characteristic import Characteristic


class GeometryType(Characteristic):
    __type__ = CLV["GeometryType"]


class Box(GeometryType):
    def __init__(self):
        self.uriRef = URIRef(CLV["Box"])


class Line(GeometryType):
    def __init__(self):
        self.uriRef = URIRef(CLV["Line"])


class Point(GeometryType):
    def __init__(self):
        self.uriRef = URIRef(CLV["Point"])


class Polygon(GeometryType):
    def __init__(self):
        self.uriRef = URIRef(CLV["Polygon"])



from rdflib import RDF, Graph
from rdflib.namespace import (DC, DCAT, DCTERMS, FOAF, OWL, PROV, RDF, RDFS,
                              SKOS, XMLNS, XSD)

from .ConceptScheme import ConceptScheme
from .ns import *
from .Thing import Thing

VERSION = (0, 0, 7)

__author__ = 'Luca Martinelli'
__email__ = 'martinelliluca98@gmail.com'
__version__ = '.'.join(map(str, VERSION))
__description__ = 'A python package to create OntoPiA RDFs.'


def createGraph():
    # Create the graph
    g = Graph()

    g.bind("xsd", XSD)
    g.bind("foaf", FOAF)
    g.bind("owl", OWL)
    g.bind("dc", DC)
    g.bind("xml", XMLNS)
    g.bind("dct", DCTERMS)
    g.bind("rdf", RDF)
    g.bind("rdfs", RDFS)
    g.bind("dcat", DCAT)
    g.bind("prov", PROV)
    g.bind("skos", SKOS)

    # Bind for OntoPiA
    g.bind("l0", L0)
    g.bind("transp", TRANSP)
    g.bind("ti", TI)
    g.bind("sm", SM)
    g.bind("route", ROUTE)
    g.bind("ro", RO)
    g.bind("pubc", PUBC)
    g.bind("proj", PROJ)
    g.bind("pot", POT)
    g.bind("poi", POI)
    g.bind("park", PARK)
    g.bind("mu", MU)
    g.bind("lang", LANG)
    g.bind("iot", IOT)
    g.bind("indic", INDIC)
    g.bind("her", HER)
    g.bind("culther", CULTHER)
    g.bind("cpv", CPV)
    g.bind("cpsv", CPSV)
    g.bind("cpev", CPEV)
    g.bind("cov", COV)
    g.bind("clv", CLV)
    g.bind("paths", PATHS)
    g.bind("acond", ACOND)
    g.bind("acco", ACCO)

    # Controlled vocabularies
    g.bind("eventtype", EVENTS_TYPE)
    g.bind("license", LICENSES)
    g.bind("geodist", GEO_DISTRIBUTION)
    g.bind("region", REGIONS)
    g.bind("province", PROVINCES)
    g.bind("country", COUNTRIES)
    g.bind("city", CITIES)
    g.bind("poiclass", POI_CLASSIFICATION)
    g.bind("accostar", ACCO_STAR_RATINGS)
    g.bind("accotype", ACCO_TYPES)
    g.bind("educationlevel", PERSON_EDULEVEL)
    g.bind("persontitle", PERSON_TITLE)
    g.bind("parentalrelation", PERSON_PARENTAL_REL)
    g.bind("legalstatus", ORG_LEGAL_STATUS)
    g.bind("ateco", ORG_ATECO)
    g.bind("s13", ORG_S13)
    g.bind("discipline", CUL_SUBJ)
    g.bind("culturalplace", CUL_PLACES)
    g.bind("routetype", ROUTE_TYPES)
    g.bind("authtype", AUTH_TYPES)
    g.bind("channel", EROGATION_CHANNELS)
    g.bind("interactionlevel", INTERACT_LEVEL)
    g.bind("ioservice", IO_SERVICE)
    g.bind("lifeevent", LIFE_EVENTS)
    g.bind("businessevent", BUSINESS_EVENTS)

    return g

# Serialize RDF and save it in different extensions


def saveGraph(g, fileName):
    formats = [
        {"ext": "ttl", "fmt": "turtle"},
        {"ext": "rdf", "fmt": "xml"}
    ]

    for format in formats:
        ext = format["ext"]
        fmt = format["fmt"]

        with open("{}.{}".format(fileName, ext), "w", encoding="utf-8") as fp:
            fp.write(g.serialize(format=fmt))



from .ActivityType import ActivityType
from .BalanceSheet import BalanceSheet
from .ChangeEvent import ChangeEvent
from .Closing import Closing
from .PrivateOrganization import PrivateOrganization
from .SupportUnit import SupportUnit
from .eInvoiceService import eInvoiceService
from .Employment import Employment
from .Foundation import Foundation
from .Group import Group
from .HomogeneousOrganizationalArea import HomogeneousOrganizationalArea
from .LegalStatus import LegalStatus
from .Merger import Merger
from .Office import Office
from .Organization import Organization
from .Participation import Participation
from .ParticipationType import ParticipationType
from .PrivateOrgActivityType import PrivateOrgActivityType
from .PublicOrganization import PublicOrganization
from .PublicOrgActivityType import PublicOrgActivityType
from .PublicOrganizationCategory import PublicOrganizationCategory
from .PublicPrivateOrganization import PublicPrivateOrganization
from .Replacement import Replacement
from .Scission import Scission
from .Status import Status


from rdflib import Namespace

# OntoPiA Namespaces

L0 = Namespace("https://w3id.org/italia/onto/l0/")
TRANSP = Namespace("https://w3id.org/italia/onto/Transparency/")
TI = Namespace("https://w3id.org/italia/onto/TI/")
SM = Namespace("https://w3id.org/italia/onto/SM/")
ROUTE = Namespace("https://w3id.org/italia/onto/Route/")
RO = Namespace("https://w3id.org/italia/onto/RO/")
PUBC = Namespace("https://w3id.org/italia/onto/PublicContract/")
PROJ = Namespace("https://w3id.org/italia/onto/Project/")
POT = Namespace("https://w3id.org/italia/onto/POT/")
POI = Namespace("https://w3id.org/italia/onto/POI/")
PARK = Namespace("https://w3id.org/italia/onto/PARK/")
MU = Namespace("https://w3id.org/italia/onto/MU/")
LANG = Namespace("https://w3id.org/italia/onto/Language/")
IOT = Namespace("https://w3id.org/italia/onto/IoT/")
INDIC = Namespace("https://w3id.org/italia/onto/Indicator/")
HER = Namespace("https://w3id.org/italia/onto/HER/")
CULTHER = Namespace("https://w3id.org/italia/onto/CulturalHeritage/")
CPV = Namespace("https://w3id.org/italia/onto/CPV/")
CPSV = Namespace("https://w3id.org/italia/onto/CPSV/")
CPEV = Namespace("https://w3id.org/italia/onto/CPEV/")
COV = Namespace("https://w3id.org/italia/onto/COV/")
CLV = Namespace("https://w3id.org/italia/onto/CLV/")
PATHS = Namespace("https://w3id.org/italia/onto/AtlasOfPaths/")
ACOND = Namespace("https://w3id.org/italia/onto/AccessCondition/")
ACCO = Namespace("https://w3id.org/italia/onto/ACCO/")

# OntoPiA Controlled Vocabularies

EVENTS_TYPE = Namespace(
    "https://w3id.org/italia/controlled-vocabulary/public-event-types/")
LICENSES = Namespace(
    "https://w3id.org/italia/controlled-vocabulary/licenses/")
GEO_DISTRIBUTION = Namespace(
    "https://w3id.org/italia/controlled-vocabulary/territorial-classifications/geographical-distribution/")
REGIONS = Namespace(
    "https://w3id.org/italia/controlled-vocabulary/territorial-classifications/regions/")
PROVINCES = Namespace(
    "https://w3id.org/italia/controlled-vocabulary/territorial-classifications/provinces/")
COUNTRIES = Namespace(
    "https://w3id.org/italia/controlled-vocabulary/territorial-classifications/countries/")  # Note: only /italy available
CITIES = Namespace(
    "https://w3id.org/italia/controlled-vocabulary/territorial-classifications/cities/")
POI_CLASSIFICATION = Namespace(
    "https://w3id.org/italia/controlled-vocabulary/poi-category-classification/")
ACCO_STAR_RATINGS = Namespace(
    "https://w3id.org/italia/controlled-vocabulary/classifications-for-accommodation-facilities/accommodation-star-rating/")
ACCO_TYPES = Namespace(
    "https://w3id.org/italia/controlled-vocabulary/classifications-for-accommodation-facilities/accommodation-typology/")
PERSON_EDULEVEL = Namespace(
    "https://w3id.org/italia/controlled-vocabulary/classifications-for-people/education-level/")
PERSON_TITLE = Namespace(
    "https://w3id.org/italia/controlled-vocabulary/classifications-for-people/person-title/")
PERSON_PARENTAL_REL = Namespace(
    "https://w3id.org/italia/controlled-vocabulary/classifications-for-people/parental-relationship-types/")
ORG_LEGAL_STATUS = Namespace(
    "https://w3id.org/italia/controlled-vocabulary/classifications-for-organizations/legal-status/")
ORG_ATECO = Namespace(
    "https://w3id.org/italia/controlled-vocabulary/classifications-for-organizations/ateco-2007/")
ORG_S13 = Namespace(
    "https://w3id.org/italia/controlled-vocabulary/classifications-for-organizations/S13/")
CUL_SUBJ = Namespace(
    "https://w3id.org/italia/controlled-vocabulary/classifications-for-culture/subject-disciplines/")
CUL_PLACES = Namespace(
    "https://w3id.org/italia/controlled-vocabulary/classifications-for-culture/cultural-interest-places/")
ROUTE_TYPES = Namespace(
    "https://w3id.org/italia/controlled-vocabulary/classifications-for-routes/route-types/")
AUTH_TYPES = Namespace(
    "https://w3id.org/italia/controlled-vocabulary/classifications-for-public-services/authentication-type/")
EROGATION_CHANNELS = Namespace(
    "https://w3id.org/italia/controlled-vocabulary/classifications-for-public-services/channel/")
INTERACT_LEVEL = Namespace(
    "https://w3id.org/italia/controlled-vocabulary/classifications-for-public-services/interactivity-level/")
IO_SERVICE = Namespace(
    "https://w3id.org/italia/controlled-vocabulary/classifications-for-public-services/service-input-output/")
LIFE_EVENTS = Namespace(
    "https://w3id.org/italia/controlled-vocabulary/classifications-for-public-services/life-business-event/life-event/")
BUSINESS_EVENTS = Namespace(
    "https://w3id.org/italia/controlled-vocabulary/classifications-for-public-services/life-business-event/business-event/")



from rdflib import Graph

from ..ns import *
from ..ti.TimeIndexedEvent import TimeIndexedEvent
from ..clv.Address import Address
from .Person import Person


class ResidenceInTime(TimeIndexedEvent):
    __type__ = CPV["ResidenceInTime"]

    residenceInLocation: Address = None
    isResidenceInTimeOf: Person = None

    def _addProperties(self, g: Graph):
        super()._addProperties(g)

        if self.residenceInLocation:
            g.add(self.uriRef, CPV["residenceInLocation"],
                  self.residenceInLocation.uriRef)

        if self.isResidenceInTimeOf:
            g.add(self.uriRef, CPV["isResidenceInTimeOf"],
                  self.isResidenceInTimeOf.uriRef)



from typing import List

from rdflib import Graph, Literal

from ..ns import *
from .SupportUnit import SupportUnit


class HomogeneousOrganizationalArea(SupportUnit):
    __type__ = COV["HomogeneousOrganizationalArea"]

    AOOIdentifier: List[Literal] = None

    def _addProperties(self, g: Graph):
        super()._addProperties(g)

        if self.AOOIdentifier:
            for AOOIdentifier in self.AOOIdentifier:
                g.add((self.uriRef, COV["AOOIdentifier"], AOOIdentifier))



from rdflib import Graph, Literal

from ..l0.Object import Object
from ..ns import *
from ..ti.Year import Year


class BalanceSheet(Object):
    __type__ = COV["BalanceSheet"]

    hasYear: Year = None
    totalAmount: Literal = None
    totalTaxBurden: Literal = None

    def _addProperties(self, g: Graph):
        super()._addProperties(g)

        if self.hasYear:
            g.add(self.uriRef, TI["hasYear"], self.hasYear.uriRef)

        if self.totalAmount:
            g.add(self.uriRef, COV["totalAmount"], self.totalAmount)

        if self.totalTaxBurden:
            g.add(self.uriRef, COV["totalTaxBurden"], self.totalTaxBurden)



from typing import List

from rdflib import Graph, Literal

from ..ns import *
from ..l0.Object import Object
from .UserAccount import UserAccount


class SocialMedia(Object):
    __type__ = SM["SocialMedia"]

    socialMediaName: Literal = None
    issuesAccount: List[UserAccount] = None

    def _addProperties(self, g: Graph):
        super()._addProperties(g)

        if self.socialMediaName:
            g.add((self.uriRef, SM["socialMediaName"], self.socialMediaName))

        if self.issuesAccount:
            for issuesAccount in self.issuesAccount:
                g.add((self.uriRef, SM["issuesAccount"], issuesAccount.uriRef))



from typing import List

from rdflib import Graph, Literal

from ..ns import *
from ..l0.Characteristic import Characteristic
from .MeasurementUnit import MeasurementUnit
from .MeasureType import MeasureType


class Value(Characteristic):
    __type__ = MU["Value"]

    hasMeasurementUnit: List[MeasurementUnit] = None
    hasMeasureType: MeasureType = None
    value: List[Literal] = None

    def _addProperties(self, g: Graph):
        super()._addProperties(g)

        if self.hasMeasurementUnit:
            for hasMeasurementUnit in self.hasMeasurementUnit:
                g.add((self.uriRef, MU["hasMeasurementUnit"],
                       hasMeasurementUnit.uriRef))

        if self.hasMeasureType:
            g.add((self.uriRef, MU["hasMeasureType"],
                  self.hasMeasureType.uriRef))

        if self.value:
            for value in self.value:
                g.add((self.uriRef, MU["value"], value))



from rdflib import Graph, Literal

from ..ns import *
from .EducationLevel import EducationLevel
from .Person import Person


class AlivePerson(Person):
    __type__ = CPV["AlivePerson"]

    hasLevelOfEducation: EducationLevel = None
    age: Literal = None

    def _addProperties(self, g: Graph):
        super()._addProperties(g)

        if self.hasLevelOfEducation:
            g.add(self.uriRef, CPV["hasLevelOfEducation"],
                  self.hasLevelOfEducation.uriRef)

        if self.age:
            g.add(self.uriRef, CPV["age"], self.age)



from typing import List

from rdflib import Graph

from ..ns import *
from .Entity import Entity


class Topic(Entity):
    __type__ = L0["Topic"]

    isTopicOf: List[Entity] = None

    def _addProperties(self, g: Graph):
        super()._addProperties(g)

        if self.isTopicOf:
            for isTopicOf in self.isTopicOf:
                g.add((self.uriRef, L0["isTopicOf"],
                       isTopicOf.uriRef))
